/*File name:crypt_gb.h
 *
 *Date:2014-08
 *Description:cryptology(sm3, SMS4 and random key generation)
 *
 *
 */
#ifndef CRYPT_GB_H
#define CRYPT_GB_H 1

#if DEBUG
	#define DEBUG_MODE 1
	#define _dbg(...) printf(__VA_ARGS__)

#else
	#define DEBUG_MODE 0
	#define _dbg(...)
#endif  /* DEBUG */

/* base bit  quantity*/
typedef unsigned char  BOOLEAN; 
typedef unsigned char  INT8U;/* Unsigned  8 bit quantity*/ 
typedef signed   char  INT8S;/* Signed    8 bit quantity*/ 
typedef unsigned short   INT16U;/* Unsigned 16 bit quantity*/ 
typedef signed   short   INT16S;/* Signed   16 bit quantity*/ 
typedef unsigned int  INT32U;/* Unsigned 32 bit quantity*/ 
typedef signed   int  INT32S;/* Signed   32 bit quantity*/

/* error  code */
#define NO_ERR (0)
#define PARAM_ERR (-1)

/* SMS4 */
#define MAX_KEY_INT8U (256)
#define MAX_KEY_INT32U (64)
#define MAX_EXP_KEY_INT32U (256)
#define SM4_KEY_INT8U (16)
#define SM4_KEY_INT32U (4)
#define SM4_EXP_KEY_INT32U (32)

typedef struct {
	//mode
	//0:no encrypt
	//1:sms4
	INT8U mode;
	INT32U key[MAX_KEY_INT32U];
	INT32U key_len;
	INT32U exp_key[MAX_EXP_KEY_INT32U];
	INT32U exp_key_len;
} sms4_context;

int crypt_gb_sms4_start(sms4_context *ctx, const INT8U *key, INT32U keylen, INT8U mode);
int crypt_gb_sms4_encrypt(sms4_context *ctx, INT8U *src, INT32U srclen, INT32U **dest, INT32U *destlen);
int crypt_gb_sms4_decrypt(sms4_context *ctx, INT32U *src, INT32U srclen, INT8U **dest, INT32U *destlen);

/* random key generate */
int crypt_gb_genKey(INT8U **key, int keylen);

/* sm3 */
#define SM3_DIGEST_LEN (32)
typedef struct {
	unsigned int total[2];
	unsigned int state[8];
	unsigned char buffer[64];
} sm3_context;

void tcm_sm3_starts(sm3_context *ctx );
void tcm_sm3_update(sm3_context *ctx, unsigned char *input, unsigned int length);
void tcm_sm3_finish(sm3_context *ctx, unsigned char digest[32]);
int tcm_sm3_hash(unsigned int datalen_in, unsigned char *pdata_in, unsigned char digest[32]);
int tcm_hmac(unsigned char *text, unsigned int text_len, unsigned char *key, unsigned int key_len, unsigned char digest[32]);
int tcm_kdf(unsigned char *key, int klen, unsigned char *z, int zlen);

#endif /* crypt_gb.h */
